/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;
import com.topcoder.util.config.UnknownNamespaceException;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * This is the main class of the user API. It provides the basic API, which includes maintaining a
 * list of file processors, a reporting handler and controls the way to process the file. Files
 * are processed one by one and the results are aggregated into the reporting. If you want to
 * start a new reporting you should either explicitly set the reporting or get the old reporting
 * to reset it. There are two ways to process the file: - Process it with the first matching
 * processor. - Process it with each martching processor. Refer to ExtendedFileStatistics for
 * advanced usage.
 * </p>
 *
 * <p>
 * Configuration: This class can be intialized from a configuration namespace which should be
 * preloaded. It will look at the following properties: - "processors" as a container for file
 * processors, each sub-property should contain a "classname" that specifies the fully qualified
 * class name of a FileProcessor implementation which accepts a Property as constructor argument.
 * This property is required. - "reporting" which contains a "classname" that specifies the fully
 * qualified class name of a StatsReporting implementation which accepts a Property as constructor
 * argument. This property is optional (XmlStatsReporting is the default value). - "matchall"
 * which is a boolean flag that indicates whether all matching processors will be used to process
 * the file. It is regarded true when value is either "on", "yes", "true" case-insensitively. This
 * property is optional.
 * </p>
 *
 * <p>
 * Thread Safety: This class is not thread-safe in that it provides a number of manipulation
 * methods in a non-synchronized manner. Even if calling from different threads at separated time
 * will mix the result into the same reporting, which may not be the desirable behavior.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class FileStatistics {
    /**
     * <p>
     * This property contains multiple sub-properties to specify FileProcessor implementations.
     * When the implementation is instantiated the property itself is passed in as constructor
     * argument.
     * </p>
     */
    public static final String PROCESSORS_PROPERTY_KEY = "processors";

    /**
     * <p>
     * This property contains sub-properties to specify the StatsReporting implementation. When the
     * implementation is instantiated the property itself is passed in as constructor argument.
     * </p>
     */
    public static final String REPORTING_PROPERTY_KEY = "reporting";

    /**
     * <p>
     * This property specifies whether a file is processed by the first matching processor or each
     * of the matching processor. This property is optional (default to false). Its value is
     * considered true if it's "true", "yes" or "on".
     * </p>
     */
    public static final String MATCH_ALL_PROPERTY_KEY = "matchall";

    /**
     * <p>
     * This is a sub-property in "processors"' sub-properties or "reporting" to specify the fully
     * qualified class name of the FileProcessor or StatsReporting implementation that owns a
     * constructor with a Property argument.
     * </p>
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

    /**
     * <p>
     * Constant for the parameter types to retrieve the constructor.
     * </p>
     */
    private static final Class[] PARAMETER_TYPES = new Class[] {Property.class};

    /**
     * <p>
     * Constant for the parameter types to retrieve the constructor.
     * </p>
     */
    private static final List ON_SWITCHS = Arrays.asList(new String[] {"on", "yes", "true"});

    /**
     * <p>
     * The processors aggregated. This is intialized to a hashmap and the reference will never
     * change.
     * </p>
     */
    private final Map processors = new HashMap();

    /**
     * <p>
     * The list containing all processors.  This is intialized to an ArrayList and the reference
     * will never change.
     * </p>
     */
    private final List processorList = new ArrayList();

    /**
     * <p>
     * The reporting to use. This is mutable.
     * </p>
     */
    private StatsReporting reporting = null;

    /**
     * <p>
     * A flag on whether to process the file with each matching processor.
     * </p>
     */
    private boolean matchAll = false;

    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public FileStatistics() {
        reporting = new XmlStatsReporting();
    }

    /**
     * <p>
     * Constructor to load configuration from given namespace. The namespace should be already
     * loaded. It loads the processors the reporting as well as the matchall flag.
     * </p>
     *
     * @param namespace the namespace to load from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty after trimmed
     * @throws ConfigurationException if config error occurs
     */
    public FileStatistics(String namespace) throws ConfigurationException {
        FileStatisticsHelper.checkEmpty(namespace, "namespace");

        initializeProcessors(namespace);
        initializeReporting(namespace);
        initializeMatchAll(namespace);
    }

    /**
     * Retrieves matchAll from the ginve namespace.
     *
     * @param namespace the config namespace
     *
     * @throws ConfigurationException if some error occurs
     */
    private void initializeMatchAll(String namespace) throws ConfigurationException {
        Property matchAllProperty = getProperty(namespace, MATCH_ALL_PROPERTY_KEY, false);

        if (matchAllProperty != null) {
            String matchAllSwitch = matchAllProperty.getValue();
            // turn on matchAll switch if the property value is yes, on, true
            if ((matchAllSwitch != null) && ON_SWITCHS.contains(matchAllSwitch.toLowerCase())) {
                this.matchAll = true;
            }
        }
    }

    /**
     * Retrieves reporting from the ginve namespace.
     *
     * @param namespace the config namespace
     *
     * @throws ConfigurationException if some error occurs
     */
    private void initializeReporting(String namespace)
        throws ConfigurationException {
        Property reportingProperty = getProperty(namespace, REPORTING_PROPERTY_KEY, false);

        if (reportingProperty == null) {
            // if reportingProperty does not exist, create an XmlStatsReporting
            this.reporting = new XmlStatsReporting();
        } else {
            // otherwise create the specified reporting
            this.reporting = (StatsReporting) createObject(reportingProperty, StatsReporting.class);
        }
    }

    /**
     * Retrieves processors from the ginve namespace.
     *
     * @param namespace the config namespace
     *
     * @throws ConfigurationException if some error occurs
     */
    private void initializeProcessors(String namespace)
        throws ConfigurationException {
        Property processorsProperty = getProperty(namespace, PROCESSORS_PROPERTY_KEY, true);
        Enumeration processorsEnum = processorsProperty.propertyNames();

        // for each sub-property create a FileProcessor insatnce
        while (processorsEnum.hasMoreElements()) {
            String name = (String) processorsEnum.nextElement();
            Property processorProperty = processorsProperty.getProperty(name);

            // delegate to createObject method, and add the new processor
            this.addFileProcessor((FileProcessor) createObject(processorProperty,
                    FileProcessor.class));
        }

        // throw an exception if no processor created
        if (this.processorList.isEmpty()) {
            throw new ConfigurationException(PROCESSORS_PROPERTY_KEY
                + "does not have any sub-property.");
        }
    }

    /**
     * Creates an object instance from given property via reflection.
     *
     * @param property the property to retrieve
     * @param targetClass the name of the sub-property
     *
     * @return the created object
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static Object createObject(Property property, Class targetClass)
        throws ConfigurationException {
        // get the class name
        String className = getPropertyString(property, CLASSNAME_PROPERTY_KEY);

        try {
            Class objectClass = Class.forName(className);

            if (!targetClass.isAssignableFrom(objectClass)) {
                throw new ConfigurationException("the class does not implement " + targetClass);
            }

            // retrieve the public constructor by the parameter type array
            Constructor constructor = objectClass.getConstructor(PARAMETER_TYPES);

            // create the matcher instacne via reflection
            return constructor.newInstance(new Object[] {property});
        } catch (ConfigurationException ce) {
            throw ce;
        } catch (Exception e) {
            // wrap the exception with ConfigurationException
            throw new ConfigurationException("Exception occurs while creating the instance", e);
        }
    }

    /**
     * Retrieve single string value of the specified sub-property. Some necessary validations are
     * performed.
     *
     * @param property the property to retrieve
     * @param key the name of the sub-property
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static String getPropertyString(Property property, String key)
        throws ConfigurationException {
        // get the subproperty
        Property subproperty = property.getProperty(key);

        if (subproperty == null) {
            throw new ConfigurationException(key + " property does not exits");
        }

        // get the string value
        String value = subproperty.getValue();

        // throw an exception if the value does not exist.
        if (value == null) {
            throw new ConfigurationException(key + " property does not contain any value.");
        }

        if (value.trim().length() == 0) {
            throw new ConfigurationException(key + " property is an empty string.");
        }

        return value;
    }

    /**
     * Get the specified property from given namespace.
     *
     * @param namespace the config namespace
     * @param key the property key
     * @param required whether this property is required
     *
     * @return the property retrieved
     *
     * @throws ConfigurationException if the namespace is invalid or the property does not exist if
     *         it's required
     */
    private Property getProperty(String namespace, String key, boolean required)
        throws ConfigurationException {
        Property property = null;

        try {
            property = ConfigManager.getInstance().getPropertyObject(namespace, key);
        } catch (UnknownNamespaceException une) {
            throw new ConfigurationException("the given namespace does not exits", une);
        }

        if ((property == null) && required) {
            throw new ConfigurationException(PROCESSORS_PROPERTY_KEY + " property does not exist");
        }

        return property;
    }

    /**
     * <p>
     * Add the file processor to the aggregation. If there is a processor with the same alias just
     * overwrite the old one. Note: the processor will be added to the end of the inner list.
     * </p>
     *
     * @param processor the file processor to add
     *
     * @throws NullPointerException if processor is null
     */
    public void addFileProcessor(FileProcessor processor) {
        FileStatisticsHelper.checkNull(processor, "processor");

        // add the new processor, and get the old one if it exists
        Object oldProcessor = processors.put(processor.getAlias(), processor);

        if (oldProcessor != null) {
            // if the processor with same alias exists, will remove it from the
            // inner list
            processorList.remove(oldProcessor);
        }

        processorList.add(processor);
    }

    /**
     * <p>
     * Remove the file processor with given alias. If no such process exists return null.
     * </p>
     *
     * @param alias the alias of the processor
     *
     * @return the processor just removed
     *
     * @throws NullPointerException if alias is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public FileProcessor removeFileProcessor(String alias) {
        FileStatisticsHelper.checkEmpty(alias, "alias");

        // remove the processor, and get the reference if it exists
        FileProcessor removedProcessor = (FileProcessor) processors.remove(alias);

        if (removedProcessor != null) {
            // remove the processor from the inner list if it exists
            processorList.remove(removedProcessor);
        }

        return removedProcessor;
    }

    /**
     * <p>
     * Get the file processor with given alias. If no such processor exists return null.
     * </p>
     *
     * @param alias the alias of the processor
     *
     * @return the associated file processor
     *
     * @throws NullPointerException if alias is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public FileProcessor getFileProcessor(String alias) {
        FileStatisticsHelper.checkEmpty(alias, "alias");

        return (FileProcessor) processors.get(alias);
    }

    /**
     * <p>
     * Get a list of all the processors aggregated.  The returned list is shallow a copy of the
     * inner collection.
     * </p>
     *
     * @return a list of all the processors
     */
    public List getAllFileProcessors() {
        return new ArrayList(processorList);
    }

    /**
     * <p>
     * Clear all the aggregated processors.
     * </p>
     */
    public void clearAllFileProcessors() {
        processors.clear();
        processorList.clear();
    }

    /**
     * <p>
     * Get the reporting instance.
     * </p>
     *
     * @return the stats reporting
     */
    public StatsReporting getReporting() {
        return reporting;
    }

    /**
     * <p>
     * Set the stats reporting implementation. Once the new instance is set the old results are
     * dropped.
     * </p>
     *
     * @param reporting the stats reporting implementation
     *
     * @throws NullPointerException if reporting is null
     */
    public void setReporting(StatsReporting reporting) {
        FileStatisticsHelper.checkNull(reporting, "reporting");
        this.reporting = reporting;
    }

    /**
     * <p>
     * Whether to process the file with each matching processor.
     * </p>
     *
     * @return whether to process the file with each matching processor
     */
    public boolean isMatchAll() {
        return matchAll;
    }

    /**
     * <p>
     * Set whether to process the file with each matching processor.
     * </p>
     *
     * @param matchAll whether to process the file with each matching processor
     */
    public void setMatchAll(boolean matchAll) {
        this.matchAll = matchAll;
    }

    /**
     * <p>
     * Process a single file.
     * </p>
     *
     * <p>
     * 1) If matchall is on, iterate through all file processors; 2) If matchall is off, iterate
     * until we meet the first matching process; 3) Process the file with the matched
     * processor(s); 4) Aggregate the result into reporting; 5) If no processor matches the file
     * nothing happens.
     * </p>
     *
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processing fails
     */
    public void processFile(File file) throws FileStatisticsException, IOException {
        FileStatisticsHelper.checkNull(file, "file");

        // itearate through all processors
        for (Iterator itr = processorList.iterator(); itr.hasNext();) {
            FileProcessor processor = (FileProcessor) itr.next();

            if (processor.getFileMatcher().match(file)) {
                // for each matched processor, aggregate the result
                StatsResult result = processor.process(file);
                reporting.aggregate(processor, file, result);

                if (!matchAll) {
                    // if matchAll is off, return immediately after first match
                    return;
                }
            }
        }
    }
}
