/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.util.commandline.ArgumentValidationException;
import com.topcoder.util.commandline.CommandLineUtility;
import com.topcoder.util.commandline.IllegalSwitchException;
import com.topcoder.util.commandline.Switch;
import com.topcoder.util.commandline.UsageException;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * The command line entry point of the File Statistics utility. It uses ExtendedFileStatistics to
 * fulfill the task. When argument is invalid help message should be printed.
 * </p>
 *
 * <p>
 * Arguments: java Main [-c config-file] [-n namespace] [-b base-path] [-r] [-o output-file] path
 * [path2 path3 ...] -c config-file: provides a config file to initialize from -n namespace:
 * provides a namespace to initialize from If neither is provided the default namespace is used.
 * Provided namespace should be preloaded. -b base-path: all path will be relative to it. -r:
 * whether to recursively traverse sub-directory. -o output-file: the output filename. If missing
 * output is printed to console.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless and thread-safet.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class Main {

    /**
     * <p>
     * The namespace to used if no namespace is provided.
     * </p>
     */
    public static final String NAMESPACE = "com.topcoder.file.statistics.Main";

    /**
     * Switch c takes 1 parameter. This switch is not required.
     */
    private static final String SWITCH_C = "c";

    /**
     * Switch n takes 1 parameter. This switch is not required.
     */
    private static final String SWITCH_N = "n";

    /**
     * Switch r takes no parameter. This switch is not required.
     */
    private static final String SWITCH_R = "r";

    /**
     * Switch b takes 1 parameter. This switch is not required.
     */
    private static final String SWITCH_B = "b";

    /**
     * Switch o takes 1 parameter. This switch is not required.
     */
    private static final String SWITCH_O = "o";

    /**
     * The parameters key.
     */
    private static final String PARAMETERS = "parameters";

    /**
     * <p>
     * Private constructor to disallow instantiation.
     * </p>
     */
    private Main() {
        // empty
    }

    /**
     * <p>
     * Command line entry. Refer to class level doc for options (which will be parsed by command
     * line utility).
     * </p>
     *
     * @param args the command line args
     */
    public static void main(String[] args) {
        // delegate to parseArguments method to parse the arguments
        Map arguments = parseArguments(args);

        // if the arguments is invalid, output a help message to console
        if (arguments == null) {
            printHelp();
            return;
        }

        // set the namespace
        String namespace = null;

        if (arguments.containsKey(SWITCH_N)) {
            // if switch n presents, use the specified namespace
            namespace = (String) arguments.get(SWITCH_N);
        } else {
            // otherwise use the default one
            namespace = NAMESPACE;
        }

        // if switch c presents, load the config file
        if (arguments.containsKey(SWITCH_C)) {
            try {
                ConfigManager.getInstance().add(namespace, (String) arguments.get(SWITCH_C),
                    ConfigManager.CONFIG_XML_FORMAT);
            } catch (ConfigManagerException e) {
                // if any error occurs, output an error message
                System.out.println("Error occurs while loading the config file.");
                System.out.println(e);

                return;
            }
        }

        // create an ExtendedFileStatistics instance with the namespace
        ExtendedFileStatistics fileStatistics = null;

        try {
            fileStatistics = new ExtendedFileStatistics(namespace);
        } catch (ConfigurationException ce) {
            // if any error occurs, output an error message
            System.out.println("Error occurs while creating ExtendedFileStatistics instance");
            System.out.println(ce);

            return;
        }

        // if switch b presents, set the base directory
        if (arguments.containsKey(SWITCH_B)) {
            File baseDir = new File((String) arguments.get(SWITCH_B));

            if (!baseDir.isDirectory()) {
                System.out.println("The base directory is invalid.");

                return;
            }

            fileStatistics.getReporting().reset(baseDir);
        }

        // if switch b presents, active the recursive feature
        boolean recursive = arguments.containsKey(SWITCH_R);

        List parameters = (List) arguments.get(PARAMETERS);
        // iterate through all parameters
        for (Iterator itr = parameters.iterator(); itr.hasNext();) {
            // create a File for each path
            File path = new File((String) itr.next());

            if (path.exists()) {
                try {
                    // processe the file or directory
                    if (path.isFile()) {
                        fileStatistics.processFile(path);
                    } else {
                        fileStatistics.processDirectory(path, recursive);
                    }
                } catch (FileStatisticsException fse) {
                    System.out.println("Error occurs while processing " + path);
                    System.out.println(fse);

                    return;
                } catch (IOException ioe) {
                    System.out.println("IO Error occurs while processing " + path);
                    System.out.println(ioe);

                    return;
                }
            } else {
                System.out.println(path + " does not exist.");

                return;
            }
        }

        try {
            if (arguments.containsKey(SWITCH_O)) {
                // if switchr o presents, write the report the given file
                fileStatistics.printReport(new File((String) arguments.get(SWITCH_O)));
            } else {
                // otherwis simply output to console
                fileStatistics.printReport();
            }
        } catch (IOException ioe) {
            System.out.println("IO Error occurs while writing the report");
            System.out.println(ioe);

            return;
        } catch (StatsReportingException e) {
            System.out.println("Error occurs while generating the report");
            System.out.println(e);

            return;
        }
    }

    /**
     * <p>
     * Print help messages to console.
     * </p>
     */
    private static void printHelp() {
        // the general usage
        System.out.print("Usage: java Main [-c config-file] [-n namespace] [-b base-path] [-r]");
        System.out.println(" [-o output-file] path [path2 path3 ...]");

        // the arguments
        System.out.println("Arguments:");
        System.out.println("\t-c config-file: provides a config file to initialize from");
        System.out.print("\t-n namespace: provides a namespace to initialize from. ");
        System.out.print("If neither is provided the default namespace is used. ");
        System.out.println("Provided namespace should be preloaded.");
        System.out.println("\t-b base-path: all path will be relative to it");
        System.out.println("\t-r whether to recursively traverse sub-directory");
        System.out.println(
            "\t-o output-file: the output filename. If missing output is printed to console.");
    }

    /**
     * Parses the given argument array.
     *
     * @param args the arguments to parse
     *
     * @return a map contains the arguments or null if it's invalid
     */
    private static Map parseArguments(String[] args) {
        CommandLineUtility commandLine = new CommandLineUtility(false, 1, -1);

        Switch switchC = null;
        Switch switchN = null;
        Switch switchB = null;
        Switch switchR = null;
        Switch switchO = null;

        try {
            // initialize switchs
            switchC = new Switch(SWITCH_C, false, 1, 1, null);
            switchN = new Switch(SWITCH_N, false, 1, 1, null);
            switchB = new Switch(SWITCH_B, false, 1, 1, null);
            switchR = new Switch(SWITCH_R, false, 0, 0, null);
            switchO = new Switch(SWITCH_O, false, 1, 1, null);
            commandLine.addSwitch(switchC);
            commandLine.addSwitch(switchN);
            commandLine.addSwitch(switchB);
            commandLine.addSwitch(switchR);
            commandLine.addSwitch(switchO);
        } catch (IllegalSwitchException ise) {
            // ignore here, since this never happens
        }

        try {
            commandLine.parse(args);
        } catch (ArgumentValidationException e) {
            // ignore here, since this never happens
        } catch (UsageException e) {
            // return null if error occrus
            return null;
        }

        List switches = commandLine.getValidSwitches();
        Map arguments = new HashMap();

        // add all valid switchs to the map
        for (Iterator itr = switches.iterator(); itr.hasNext();) {
            Switch sw = (Switch) itr.next();
            arguments.put(sw.getName(), sw.getValue());
        }

        // add the path list
        arguments.put(PARAMETERS, commandLine.getParameters());

        return arguments;
    }
}
