/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.util.config.Property;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * A Helper calss in matcher package.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
class MatcherHelper {
    /**
     * Private constructor to prevent this class be instantiated.
     */
    private MatcherHelper() {
        // empty
    }

    /**
     * Retrieve the specified sub-property from the give property. Some necessary validations are
     * performed.
     *
     * @param property the argument to check
     * @param key the name of the argument
     *
     * @return the String array retrieved from the given property.
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property or it is
     *         empty.
     */
    static String[] getProperty(Property property, String key)
        throws ConfigurationException {
        // check for null
        if (property == null) {
            throw new NullPointerException("property should not be null.");
        }

        Property subproperty = property.getProperty(key);

        // check whether the sub-property exists
        if (subproperty == null) {
            throw new ConfigurationException("property does not contain the given sub-property.");
        }

        // retrieve string values
        String[] values = subproperty.getValues();

        if (values == null) {
            throw new ConfigurationException("the given property does not hold any values.");
        }

        return values;
    }

    /**
     * Checks whether the given Object is null.
     *
     * @param arg the argument to check
     * @param name the name of the argument
     *
     * @throws NullPointerException if the given Object is null
     */
    static void checkNull(Object arg, String name) {
        if (arg == null) {
            throw new NullPointerException(name + " should not be null.");
        }
    }

    /**
     * Checks whether the given String is empty.
     *
     * @param arg the argument to check
     * @param name the name of the argument
     *
     * @throws NullPointerException if the given String is null
     * @throws IllegalArgumentException if the given String is empty
     */
    static void checkEmpty(String arg, String name) {
        if (arg == null) {
            throw new NullPointerException(name + " should not be null.");
        }

        if (arg.trim().length() == 0) {
            throw new IllegalArgumentException(name + " should not be empty.");
        }
    }

    /**
     * Checks whether the given String list is valid.
     *
     * @param list the list to check
     * @param name the name of the argument
     *
     * @throws NullPointerException if the list is null
     * @throws IllegalArgumentException if the list is empty or contains null, non-String instance
     *         or contains empty string (after trimmed)
     */
    static void checkStringList(List list, String name) {
        // check for null
        checkNull(list, name);

        // check for empty
        if (list.size() == 0) {
            throw new IllegalArgumentException(name + " should not be empty list.");
        }

        // check all elements.
        for (Iterator itr = list.iterator(); itr.hasNext();) {
            Object element = itr.next();

            if (element == null) {
                throw new IllegalArgumentException(name + " contains null element.");
            } else if (!(element instanceof String)) {
                throw new IllegalArgumentException(name + " contains non-String instance.");
            } else if (((String) element).trim().length() == 0) {
                throw new IllegalArgumentException(name + " contains empty string.");
            }
        }
    }

    /**
     * Extract a string list form given Property.
     *
     * @param property the property to extract
     * @param key the key of the property
     *
     * @return the string list
     *
     * @throws ConfigurationException if property does not hold a given sub-property or it does not
     *         contain any values or any value is empty after trimmed
     */
    static List extractStringList(Property property, String key)
        throws ConfigurationException {
        // get a string array
        String[] values = getProperty(property, key);

        // checks for empty strings
        for (int i = 0; i < values.length; ++i) {
            if (values[i].trim().length() == 0) {
                throw new ConfigurationException("the property contains empty string value");
            }
        }

        // convert array to list and return
        return Arrays.asList(values);
    }
}
