/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;
import com.topcoder.util.config.Property;
import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * MimeMatcher concrete implementation that matches file types with one or more mimes (as
 * configured in Magic Numbers).
 * </p>
 *
 * <p>
 * Configuration: This class can be constructed from a Property instance (reflection contract). The
 * supported property is "mimes" which is potentially multi-valued.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcher extends FileTypeMatcher {
    /**
     * <p>
     * Configuration key that contains multiple file mimes to match. Empty mimes are not allowed
     * even if they are valid in the Magic Numbers component.
     * </p>
     */
    public static final String MIMES_PROPERTY_KEY = "mimes";

    /**
     * <p>
     * A list of file mimes (String) to match. This list is initialized in constructor and is
     * immutable.
     * </p>
     */
    private final List mimes;

    /**
     * <p>
     * Constructor with single file mime.
     * </p>
     *
     * @param mime the file mime
     *
     * @throws NullPointerException if mime is null
     * @throws IllegalArgumentException if mime is empty
     */
    public MimeMatcher(String mime) {
        MatcherHelper.checkEmpty(mime, "mime");
        mimes = new ArrayList();
        mimes.add(mime);
    }

    /**
     * <p>
     * Constructor with multiple file mimes.
     * </p>
     *
     * @param mimes the file mimes to match
     *
     * @throws NullPointerException if mimes is null
     * @throws IllegalArgumentException if mimes is empty or contains null, non-String instance or
     *         contains empty string (after trimmed)
     */
    public MimeMatcher(List mimes) {
        // check the string list
        MatcherHelper.checkStringList(mimes, "mimes");

        // make a shallow copy
        this.mimes = new ArrayList(mimes);
    }

    /**
     * <p>
     * Constructor that loads mimes from a property. It looks for an "mimes" sub-property and
     * extract multiple values.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not hold a mimes sub-property or it does not
     *         contain any values or any value is empty after trimmed
     */
    public MimeMatcher(Property property) throws ConfigurationException {
        // extract a mime string list form given property
        this.mimes = MatcherHelper.extractStringList(property, MIMES_PROPERTY_KEY);
    }

    /**
     * <p>
     * Check whether any of the mimes match the type. The matching is case-sensitive.
     * </p>
     *
     * @param type the type to match
     *
     * @return whether the type matches
     */
    protected boolean match(FileType type) {
        // check for null
        MatcherHelper.checkNull(type, "type");

        // return whether the mime name is in the inner list
        return mimes.contains(type.getMime());
    }
}
