/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.util.config.Property;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;


/**
 * <p>
 * This is a FileNameMatcher concrete implementation that matches file names with one or more
 * regular expression.
 * </p>
 *
 * <p>
 * Configuration: This class can be constructed from a Property instance (reflection contract). The
 * supported property is "patterns" which is potentially multi-valued.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class RegexMatcher extends FileNameMatcher {
    /**
     * <p>
     * Configuration key that contains multiple regular expressions to match. The regex are
     * compiled when loaded and invalid pattern will result in a ConfigurationException.
     * </p>
     */
    public static final String PATTERNS_PROPERTY_KEY = "patterns";

    /**
     * <p>
     * A list of compiled regular expressionss (Pattern) to match. This list is immutable.
     * </p>
     */
    private final List patterns;

    /**
     * <p>
     * Creates a RegexMatcher instance with single pattern.
     * </p>
     *
     * @param pattern the regex pattern
     *
     * @throws NullPointerException if pattern is null
     */
    public RegexMatcher(Pattern pattern) {
        if (pattern == null) {
            throw new NullPointerException("pattern should not be null.");
        }

        // add to inner list
        patterns = new ArrayList();
        patterns.add(pattern);
    }

    /**
     * <p>
     * Creates a RegexMatcher instance with multiple regular expression patterns.
     * </p>
     *
     * @param patterns the patterns to match
     *
     * @throws NullPointerException if patterns is null
     * @throws IllegalArgumentException if patterns is empty or contains null or non-Pattern
     *         instance
     */
    public RegexMatcher(List patterns) {
        // check for null
        if (patterns == null) {
            throw new NullPointerException("patterns should not be null.");
        }

        // check for empty
        if (patterns.size() == 0) {
            throw new IllegalArgumentException("patterns should not be empty list.");
        }

        // check for non-String element.
        for (Iterator itr = patterns.iterator(); itr.hasNext();) {
            Object pattern = itr.next();

            if (pattern == null) {
                throw new IllegalArgumentException("pattern contains null element.");
            } else if (!(pattern instanceof Pattern)) {
                throw new IllegalArgumentException("pattern contains non-Pattern instance.");
            }
        }

        this.patterns = new ArrayList(patterns);
    }

    /**
     * <p>
     * Constructor that loads patterns from a property. An "extensions" sub-property should exist
     * in the given property. Each pattern is compiled when loaded.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property or it is
     *         empty or it contains invalid regular expresssion
     */
    public RegexMatcher(Property property) throws ConfigurationException {
        String[] patternStrings = MatcherHelper.getProperty(property, PATTERNS_PROPERTY_KEY);

        this.patterns = new ArrayList();

        for (int i = 0; i < patternStrings.length; ++i) {
            try {
                patterns.add(Pattern.compile(patternStrings[i]));
            } catch (PatternSyntaxException pse) {
                throw new ConfigurationException(
                    "the property contains invalid regular expresssion - " + patternStrings[i]);
            }
        }
    }

    /**
     * <p>
     * Check whether any of the patterns match the filename.
     * </p>
     *
     * @param filename the filename to match
     *
     * @return whether the filename matches
     */
    protected boolean match(String filename) {
        for (Iterator itr = patterns.iterator(); itr.hasNext();) {
            Pattern pattern = (Pattern) itr.next();

            // return true immediatelly when a pattern matches
            if (pattern.matcher(filename).matches()) {
                return true;
            }
        }

        // return false if none of the patterns matches.
        return false;
    }
}
