/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.processor;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.util.config.Property;
import java.lang.reflect.Constructor;


/**
 * <p>
 * A base implementation that can be used for FileProcessors. It provides alias and matcher
 * support, including loading them from configuration file. It serves TextLineCounter and
 * CStyleLineCounter as well as can be genericly used by future implementations.
 * </p>
 *
 * <p>
 * Configuration: This class can load configuration from a Property. The following properties are
 * supported: - "alias" string value of the alias for this processor. This property is required
 * and can not be empty. - "matcher" which specifies a file matcher. It contains a "classname"
 * which specifies the fully qualified class name of a FileMatcher implementation that owns a
 * constructor with a property argument.
 * </p>
 *
 * <p>
 * Thread Safety: This class is immutable and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileProcessorBase implements FileProcessor {
    /**
     * <p>
     * This property holds the alias for the file processor. It is required and can not be empty.
     * </p>
     */
    public static final String ALIAS_PROPERTY_KEY = "alias";

    /**
     * <p>
     * This property contains sub-properties to specify the attached FileMatcher implementation.
     * When the implementation is instantiated the property itself is passed in as constructor
     * argument. If not specified AnyFileMatcher is used.
     * </p>
     */
    public static final String FILE_MATCHER_PROPERTY_KEY = "matcher";

    /**
     * <p>
     * This is a sub-property in "matcher" to specify the fully qualified class name of the
     * FileMatcher implementation that owns a constructor with a Property argument.
     * </p>
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

    /**
     * <p>
     * Constant for the parameter types to retrieve the constructor.
     * </p>
     */
    private static final Class[] PARAMETER_TYPES = new Class[] {Property.class};

    /**
     * <p>
     * This is the alias for the filr processor. It is set in constructor and never changed.
     * </p>
     */
    private final String alias;

    /**
     * <p>
     * This is the file matcher for the filr processor (and determines if a file can/should be
     * processed). It is set in constructor and never changed.
     * </p>
     */
    private final FileMatcher matcher;

    /**
     * <p>
     * Constructor with both alias and matcher.
     * </p>
     *
     * @param alias the alias for the processor
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    protected FileProcessorBase(String alias, FileMatcher matcher) {
        // check arguments
        if (alias == null) {
            throw new NullPointerException("alias should not be null.");
        }

        if (alias.trim().length() == 0) {
            throw new IllegalArgumentException("alias should not be empty.");
        }

        if (matcher == null) {
            throw new NullPointerException("matcher should not be null.");
        }

        // initialize inner fields.
        this.alias = alias;
        this.matcher = matcher;
    }

    /**
     * <p>
     * This constructor loads configuration from the property. The non-empty alias and non-null
     * file matcher are required. Any error will be wrapped in a ConfigurationException.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    protected FileProcessorBase(Property property) throws ConfigurationException {
        if (property == null) {
            throw new NullPointerException("property should not be null.");
        }

        this.alias = getPropertyString(property, ALIAS_PROPERTY_KEY);
        this.matcher = createMatcher(getProperty(property, FILE_MATCHER_PROPERTY_KEY));
    }

    /**
     * Retrieve specified sub-property from given property. Some necessary validations are
     * performed.
     *
     * @param property the property to retrieve
     * @param key the name of the sub-property
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static Property getProperty(Property property, String key)
        throws ConfigurationException {
        // check for null
        if (property == null) {
            throw new NullPointerException("property should not be null.");
        }

        Property subproperty = property.getProperty(key);

        // check whether the sub-property exists
        if (subproperty == null) {
            throw new ConfigurationException("property does not contain the given sub-property - " + key);
        }

        return subproperty;
    }

    /**
     * Retrieve single string value of the specified sub-property. Some necessary validations are
     * performed.
     *
     * @param property the property to retrieve
     * @param key the name of the sub-property
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static String getPropertyString(Property property, String key)
        throws ConfigurationException {
        // get the subproperty
        Property subproperty = getProperty(property, key);

        // get the string value
        String value = subproperty.getValue();

        // throw an exception if the value does not exist.
        if (value == null) {
            throw new ConfigurationException(key + " property does not contain any value.");
        }

        if (value.trim().length() == 0) {
            throw new ConfigurationException(key + " property is an empty string.");
        }

        return value;
    }

    /**
     * Creates a FileMatcher instance from given property via reflection.
     *
     * @param property the property to retrieve
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static FileMatcher createMatcher(Property property)
        throws ConfigurationException {
        // get the class name
        String className = getPropertyString(property, CLASSNAME_PROPERTY_KEY);

        try {
            // retrieve the public constructor by the parameter type array
            Constructor matcherConstructor = Class.forName(className).getConstructor(PARAMETER_TYPES);

            // create the matcher instacne via reflection
            return (FileMatcher) matcherConstructor.newInstance(new Object[] {property});
        } catch (Exception e) {
            // wrap the exception with ConfigurationException
            throw new ConfigurationException("Exception occurs while creating the instance", e);
        }
    }

    /**
     * <p>
     * Get the alias for the file processor.
     * </p>
     *
     * @return the alias of the file processor.
     */
    public String getAlias() {
        return alias;
    }

    /**
     * <p>
     * Get the file matcher for the processor.
     * </p>
     *
     * @return the file matcher of the file processor.
     */
    public FileMatcher getFileMatcher() {
        return matcher;
    }
}
