/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import junit.framework.TestCase;
import java.io.File;


/**
 * <p>
 * Unit test cases for FileStatistics.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileStatisticsTests extends TestCase {
    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files";

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.unittest";

    /**
     * An Metric instance to test.
     */
    private Metric line = null;

    /**
     * An FileStatistics instance to test.
     */
    private FileStatistics fileStatistics = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor1 = new CStyleLineCounter("java", new ExtensionMatcher("java"));

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor2 = new TextLineCounter("text", new AnyFileMatcher());

    /**
     * Creates an FileStatistics instance.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        fileStatistics = new FileStatistics();

        line = (Metric) processor1.getSupportedMetrics().get(0);
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(String).
     * </p>
     */
    public void testFileStatistics1() {
        fileStatistics = new FileStatistics();
        assertTrue("FileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertTrue("Unable to instantiate FileStatistics.",
            fileStatistics.getAllFileProcessors().isEmpty());
        assertFalse("Unable to instantiate FileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(String) with null argument. NullPointerException
     * should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics2() throws Exception {
        try {
            new FileStatistics(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(String) with empty string.
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics3() throws Exception {
        try {
            new FileStatistics(" ");
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics4() throws Exception {
        fileStatistics = createFileStatistics("config1.xml", NAMESPACE);

        assertTrue("FileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate FileStatistics.", 2,
            fileStatistics.getAllFileProcessors().size());
        assertTrue("Unable to instantiate FileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics5() throws Exception {
        fileStatistics = createFileStatistics("config2.xml", NAMESPACE);

        assertTrue("FileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate FileStatistics.", 1,
            fileStatistics.getAllFileProcessors().size());
        assertTrue("Unable to instantiate FileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics6() throws Exception {
        fileStatistics = createFileStatistics("config3.xml", NAMESPACE);

        assertTrue("FileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate FileStatistics.", 2,
            fileStatistics.getAllFileProcessors().size());
        assertFalse("Unable to instantiate FileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics7() throws Exception {
        fileStatistics = createFileStatistics("config4.xml", NAMESPACE);

        assertTrue("FileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate FileStatistics.", 7,
            fileStatistics.getAllFileProcessors().size());

        assertFalse("Unable to instantiate FileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Tests the constructor against invalid configurations.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics8() throws Exception {
        checkInvalidConfig("invalid1.xml");
        checkInvalidConfig("invalid2.xml");
        checkInvalidConfig("invalid3.xml");
        checkInvalidConfig("invalid4.xml");
        checkInvalidConfig("invalid5.xml");
        checkInvalidConfig("invalid6.xml");
        checkInvalidConfig("invalid7.xml");
    }

    /**
     * <p>
     * Verifies the constructor - FileStatistics(String). ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testFileStatistics9() throws Exception {
        try {
            new FileStatistics(NAMESPACE);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Creates a FileStatistics instance with given config namespace.
     * </p>
     *
     * @param configFile the config file
     * @param namespace the config namespace
     *
     * @return the ExtendedFileStatistics instance
     *
     * @throws Exception to JUnit
     */
    private FileStatistics createFileStatistics(String configFile, String namespace)
        throws Exception {
        TestHelper.loadConfigFile(configFile, namespace);

        FileStatistics statistics = new FileStatistics(namespace);
        TestHelper.removeNamespace(namespace);

        return statistics;
    }

    /**
     * Test the constructor against invalid configuration.
     *
     * @param configFile the invalid config file.
     *
     * @throws Exception to JUnit
     */
    private void checkInvalidConfig(String configFile)
        throws Exception {
        TestHelper.loadConfigFile(configFile, NAMESPACE);

        try {
            new FileStatistics(NAMESPACE);
            fail("ConfigurationException should be thrown.");
        } catch (ConfigurationException e) {
            // success
        } finally {
            TestHelper.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Test addFileProcessor method with null argument. NullPointerException should be thrown.
     * </p>
     */
    public void testAddFileProcessor1() {
        try {
            this.fileStatistics.addFileProcessor(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test addFileProcessor method. The order should be maintained.
     * </p>
     */
    public void testAddFileProcessor2() {
        fileStatistics.addFileProcessor(processor1);
        assertEquals("processor1 should be added", processor1,
            fileStatistics.getAllFileProcessors().get(0));

        fileStatistics.addFileProcessor(processor2);
        assertEquals("processor2 should be added", processor2,
            fileStatistics.getAllFileProcessors().get(1));

        fileStatistics.addFileProcessor(processor1);
        assertEquals("the order is wrong", processor2, fileStatistics.getAllFileProcessors().get(0));
        assertEquals("the order is wrong", processor1, fileStatistics.getAllFileProcessors().get(1));
    }

    /**
     * <p>
     * Test removeFileProcessor method with null argument. NullPointerException should be thrown.
     * </p>
     */
    public void testRemoveFileProcessor1() {
        try {
            this.fileStatistics.removeFileProcessor(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test removeFileProcessor method.
     * </p>
     */
    public void testRemoveFileProcessor2() {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        fileStatistics.removeFileProcessor(processor1.getAlias());
        assertEquals("processor1 should be removed", 1, fileStatistics.getAllFileProcessors().size());
        assertEquals("processor1 should be removed", processor2,
            fileStatistics.getAllFileProcessors().get(0));

        fileStatistics.removeFileProcessor(processor2.getAlias());
        assertEquals("processor2 should be removed", 0, fileStatistics.getAllFileProcessors().size());
    }

    /**
     * <p>
     * Test getAllFileProcessors method. The order should be maintained.
     * </p>
     */
    public void testGetAllFileProcessors() {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);
        fileStatistics.addFileProcessor(processor1);

        assertEquals("the size should be 2", 2, fileStatistics.getAllFileProcessors().size());
        assertEquals("the order is wrong", processor2, fileStatistics.getAllFileProcessors().get(0));
        assertEquals("the order is wrong", processor1, fileStatistics.getAllFileProcessors().get(1));
    }

    /**
     * <p>
     * Test clearAllFileProcessors method.
     * </p>
     */
    public void testClearAllFileProcessors() {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);
        fileStatistics.clearAllFileProcessors();

        assertTrue("the size should be 0", fileStatistics.getAllFileProcessors().isEmpty());
    }

    /**
     * <p>
     * Test getReporting method. The order should be maintained.
     * </p>
     */
    public void testGetReporting() {
        StatsReporting reporting = new BasicStatsReporting();
        fileStatistics.setReporting(reporting);

        assertEquals("reporting should be returned", reporting, fileStatistics.getReporting());
    }

    /**
     * <p>
     * Test setReporting method with null argument. NullPointerException should be thrown.
     * </p>
     */
    public void testSetReporting1() {
        try {
            this.fileStatistics.setReporting(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test setReporting method. The order should be maintained.
     * </p>
     */
    public void testSetReporting2() {
        StatsReporting reporting = new BasicStatsReporting();
        fileStatistics.setReporting(reporting);

        assertEquals("reporting should be set", reporting, fileStatistics.getReporting());
    }

    /**
     * <p>
     * Test getReporting method. The order should be maintained.
     * </p>
     */
    public void testGetAndSetMatchAll() {
        fileStatistics.setMatchAll(true);
        assertTrue("true is expected", fileStatistics.isMatchAll());
        fileStatistics.setMatchAll(false);
        assertFalse("false is expected", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Test processFile method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile1() throws Exception {
        try {
            this.fileStatistics.processFile(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processFile method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile2() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processFile(new File(BASE_PATH, "test.java"));

        assertEquals("only processor1 should be used", 3L,
            reporting.getFileStats().get("test.java").getStat(line));

        fileStatistics.setMatchAll(true);
        reporting.reset(new File(BASE_PATH));
        fileStatistics.processFile(new File(BASE_PATH, "test.java"));

        assertEquals("both processors should be used", 9L,
            reporting.getFileStats().get("test.java").getStat(line));
    }

    /**
     * <p>
     * Test processFile method. Process multiple files.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile3() throws Exception {
        fileStatistics = createFileStatistics("config3.xml", NAMESPACE);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processFile(new File(BASE_PATH, "test.java"));
        fileStatistics.processFile(new File(BASE_PATH, "test.txt"));
        fileStatistics.processFile(new File(BASE_PATH, "dir/test.java"));
        fileStatistics.processFile(new File(BASE_PATH, "dir/test.txt"));

        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("test.txt").getStat(line));
        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("dir\\test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("dir\\test.txt").getStat(line));
    }
}
