/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;
import com.topcoder.util.config.Property;

/**
 * <p>
 * TextLineCounter counts lines for text files. It supports a single metric of "Line Count".
 * </p>
 *
 * <p>
 * Configuration: It delegates configuration to base class without any addition.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless while its base class is thread-safe (thus it is
 * thread-safe too).
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class TextLineCounter extends FileProcessorBase {
    /**
     * <p>
     * The name of the metric used for this processor.
     * </p>
     */
    public static final String NAME = "Line Count";

    /**
     * <p>
     * The display name of the metric used for this processor.
     * </p>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <p>
     * The supported metric instance.
     * </p>
     */
    private static final Metric metric = new Metric(NAME, DISPLAY_NAME, "The number of lines");

    /**
     * <p>
     * The list containing the supported metric.
     * </p>
     */
    private static final List supportedMetrics = Arrays.asList(new Metric[] {metric});

    /**
     * <p>
     * Constructor with both alias and matcher.
     * </p>
     *
     * @param alias the alias
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public TextLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <p>
     * This constructor loads configuration from the property. The non-empty alias and non-null
     * file matcher are required. Any error will be wrapped in a ConfigurationException.
     * </p>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public TextLineCounter(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * Return a list contains single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </p>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        return supportedMetrics;
    }

    /**
     * <p>
     * Count the lines in text file.
     * </p>
     *
     * <p>
     * 1) Create a text stream. 2) Count the lines until EOF is met. 3) Close the stream. 4) Create
     * a StatsResult and place the line count under the specified metric.
     * </p>
     *
     * @param file the file to process
     *
     * @return the line count result
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public StatsResult process(File file) throws IOException {
    	// check for null
    	if (file == null) {
    		throw new NullPointerException("file should not be null");
    	}

    	// open the file as a BufferedReader
    	BufferedReader reader = new BufferedReader(new FileReader(file));

    	long lines = 0;
    	// count the lines
    	while (reader.readLine() != null) {
    		lines++;
    	}

    	reader.close();

    	// create a StatsResult instance, and set the metric
    	StatsResult result = new StatsResult();
    	result.setStat(metric, lines);
        return result;
    }
}
