/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * StatsResult is container for multiple results, each of which are mapped from a Metric to a Long.
 * This class wraps some map operations as well as provides a way to clone and aggregate results.
 * </p>
 *
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsResult {
    /**
     * <p>
     * The results mapping from metric to the statistical quantity (Long). This reference is never
     * changed.
     * </p>
     */
    private final Map results;

    /**
     * <p>
     * Creates a StatsResult instance.
     * </p>
     */
    public StatsResult() {
        results = new HashMap();
    }

    /**
     * <p>
     * Create a result based on another result. This acts as a way to clone the instance.
     * </p>
     *
     * @param result the result to clone from
     *
     * @throws NullPointerException if result is null
     */
    public StatsResult(StatsResult result) {
        FileStatisticsHelper.checkNull(result, "result");
        results = new HashMap(result.results);
    }

    /**
     * <p>
     * Set a certain stat (associate metric with value). This method will overwrite old values
     * associated.
     * </p>
     *
     * @param metric the metric to associate with
     * @param result the quantity
     *
     * @throws NullPointerException if metric is null
     */
    public void setStat(Metric metric, long result) {
        // check for null
        FileStatisticsHelper.checkNull(metric, "metric");

        results.put(metric, new Long(result));
    }

    /**
     * <p>
     * Get the quantity associated with the metric.
     * </p>
     *
     * @param metric the metric associated with value
     *
     * @return the quantity
     *
     * @throws NullPointerException if metric is null
     * @throws IllegalArgumentException if no value associated
     */
    public long getStat(Metric metric) {
        // check for null
        FileStatisticsHelper.checkNull(metric, "metric");


        Long value = (Long) results.get(metric);

        // if no value associated, throw an exception
        if (value == null) {
            throw new IllegalArgumentException("No value associated with the given metric.");
        }

        return value.longValue();

    }

    /**
     * <p>
     * Get a list of all the associated metrics.
     * </p>
     *
     * @return a list of associated metrics
     */
    public List getMetrics() {

        // return a shallow copy of the inner keys
        return new ArrayList(results.keySet());
    }

    /**
     * <p>
     * Aggregate the result into the current result.
     * </p>
     *
     * @param result the result to aggregate from
     *
     * @throws NullPointerException if result is null
     */
    public void aggregate(StatsResult result) {
        // check for null
        FileStatisticsHelper.checkNull(result, "result");

        // iterate all metrics
        for (Iterator itr = result.results.entrySet().iterator(); itr.hasNext();) {
            Map.Entry entry = (Map.Entry) itr.next();
            Metric metric = (Metric) entry.getKey();
            Long value = (Long) entry.getValue();

            Long currentValue = (Long) this.results.get(metric);
            if (currentValue == null) {
                // add the metric, if it does not exist in the inner map
                this.results.put(metric, value);
            } else {
                // otherwise, set the sum as the new value
                this.results.put(metric, new Long(value.longValue() + currentValue.longValue()));
            }
        }
    }
}
