/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.util.config.Property;


/**
 * <p>
 * This class provides a StatsReporting implementation base. It aggregates results from each
 * processor and classifies them based on single file, directory or alias. It also provides an
 * overall stats. It defers the actual rendering logic to concrete implementations
 * (BasicStatsReporting and XmlStatsReporting).
 * </p>
 *
 * <p>
 * Configuration: This class has a constructor with Property for extension purpose. There is no
 * real logic in this release.
 * </p>
 *
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractStatsReporting implements StatsReporting {
	
	/**
	 * <p>
	 * This properties specifies one or more custom path separators to identify in addition to the default path separator.
	 * It is optional. But if it exists the value should contain no duplicates. It's user's responsibility not to use a valid
	 * character in potential file names.
	 * </p>
	 * 
	 * 
	 * @since 2.0c
	 */
	public static final String PATH_SEPARATORS_PROPERTY_KEY = "path_separators";
	    
    /**
     * <p>
     * The stats mapped to each file. The file path is relative to base dir or absolute if no base
     * dir. Its reference never changes.
     * </p>
     */
    private final StatsCollection fileStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each directory. The dir path is relative to base dir or absolute if no
     * base dir. Its reference never changes.
     * </p>
     */
    private final StatsCollection directoryStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each alias.
     * </p>
     */
    private final StatsCollection aliasStats = new StatsCollection();

    /**
     * <p>
     * The overall stats. It is created anew when reporting resets.
     * </p>
     */
    private StatsResult overallStats = new StatsResult();

    /**
     * <p>
     * The base dir for the reporting. This variable is nullable (meaning all paths are absolute).
     * </p>
     */
    private File baseDirectory = null;

    /**
     * <p>
     * The base path string. Retrieved from baseDirectory.
     * </p>
     */
    private String basePath = null;
    

    /**
     * <p>
     * This field contains one or more path separators (each of which is a char). It is initialized to ">" and has both
     * getter and setter. Developer can choose more efficient data structure.
     * </p>
     * 
     * 
     * @since 2.0c
     */
    private String pathSeparators = ">";
    
    /**
     * <p>
     * A set contians all characters in pathSeparators.
     * </p>
     * 
     * 
     * @since 2.0c
     */
    private Set charSet = new HashSet();
    


    /**
     * <p>
     * Default constructor.
     * </p>
     */
    protected AbstractStatsReporting() {
        // empty
    }

    /**
     * <p>
     * Constructor with property.
     * </p>
     * <p>
     * Change Log: Since version 2.0c this constructor loads the optional "path_separators" and validates it if it exists.
     * </p>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if path_separators property is invalid(contains duplicates) 
     */
    protected AbstractStatsReporting(Property property) throws ConfigurationException {
        if (property == null) {
            throw new NullPointerException("property should not be null");
        }
        
        // retieve the separators from the given property
        Property separatorsProperty = property.getProperty(PATH_SEPARATORS_PROPERTY_KEY);
        if (separatorsProperty != null) {
        	String pathSeparators = separatorsProperty.getValue();
        	if (separatorsProperty != null) {
                // check for duplicate characters.
                if (!isValidPathSeparators(pathSeparators)) {
                	throw new ConfigurationException("pathSeparators contains duplicates");
                }                      		
        	}
        }
        
    }

    /**
     * <p>
     * Aggregate a result into the reporting.
     * </p>
     *
     * <p>
     * 1) Get the file name. If base dir exists, strip the base portion off. 2) Aggregate the
     * result against file, each path it is in, alias and the overall stats. 3) Notice the dir
     * stats does not need to include the base dir (since it's same as overall).
     * </p>
     * <p>
     * Change Log: The signature is updated since version 2.0c. The implementation should take both the default path separator
     * and the custom separators configured into account.
     * </p> 
     *
     * @param processor the processor that generated the result
     * @param file the file that is processed
     * @param result the result
     *
     * @throws NullPointerException if processor or file or result is null
     * @throws IllegalArgumentException if filePath is empty after trimmed
     * @throws StatsReportingException if file is not in base dir
     */
    public void aggregate(FileProcessor processor, String filePath, StatsResult result)
        throws StatsReportingException {

        if (processor == null) {
            throw new NullPointerException("processor should not be null");
        }
        if (filePath == null) {
            throw new NullPointerException("file should not be null");
        }
        if (filePath.trim().length() == 0) {
            throw new IllegalArgumentException("file should not be empty");
        }
        if (result == null) {
            throw new NullPointerException("result should not be null");
        }

        // strip base directory from given file
        String fileName = stripBaseDirectory(filePath);

        // update the file stats
        fileStats.add(fileName, result);

        String directoryName = getParent(fileName);
        if (directoryName != null) {
            // update the directory stats, if the file is not the base directory
            directoryStats.add(directoryName, result);
        }

        // update the alias stats
        aliasStats.add(processor.getAlias(), result);

        // update the overall stats
        overallStats.aggregate(result);
    }


    /**
     * Strips the base directory from the given file.
     *
     * @param file the file to strip
     * @throws StatsReportingException if file is invalid or not in base dir
     * @return the stripped path
     */
    private String stripBaseDirectory(String file) throws StatsReportingException {

        // return the absolute path if baseDirectory is null
        if (baseDirectory == null) {
            return file;
        }

        if (!file.startsWith(basePath)) {
            // throw an exception if the file is not in base dir
            throw new StatsReportingException("The file is not in base dir.");
        }

        String name = file.substring(basePath.length());
        if (name.length() == 0) {
            // throw an exception if the file is the base dir
            throw new StatsReportingException("the file can not be base directory");
        }

        return name;
    }

    /**
     * <p>
     * Reset the reporting. Base dir can be null when absolute paths are desirable.
     * </p>
     *
     * <p>
     * 1) Clear all the collections. 2) Create a new overall StatsResult. 3) Set the base dir.
     * </p>
     *
     * @param baseDirectory the base dir to use
     *
     * @throws IllegalArgumentException if baseDirectory is not a valid dir
     */
    public void reset(File baseDirectory) {

        // reset base directory
        if (baseDirectory == null) {
            this.basePath = null;
            this.baseDirectory = null;
        } else {
            if (!baseDirectory.isDirectory()) {
                throw new IllegalArgumentException("baseDirectory is not a valid dir");
            }
            this.basePath = baseDirectory.getAbsoluteFile() + File.separator;
            this.baseDirectory = baseDirectory;
        }

        // clear all stats
        fileStats.clear();
        directoryStats.clear();
        aliasStats.clear();

        // reset overall stats
        overallStats = new StatsResult();
    }

    /**
     * <p>
     * Get the file stats.
     * </p>
     *
     * @return the file stats
     */
    public StatsCollection getFileStats() {
        return fileStats;
    }

    /**
     * <p>
     * Get the directory stats.
     * </p>
     *
     * @return the directory stats
     */
    public StatsCollection getDirectoryStats() {
        return directoryStats;
    }

    /**
     * <p>
     * Get the alias stats.
     * </p>
     *
     * @return the alias stats
     */
    public StatsCollection getAliasStats() {
        return aliasStats;
    }

    /**
     * <p>
     * Get a copy of overall stats.
     * </p>
     *
     * @return the overall stats
     */
    public StatsResult getOverallStats() {
        return new StatsResult(overallStats);
    }

    /**
     * <p>
     * Get the base dir.
     * </p>
     *
     * @return the base dir
     */
    public File getBaseDirectory() {
        return baseDirectory;
    }
    

    /**
     * <p>
     * Get the custom path separators (that do not include the default path separator in the system).
     * </p>
     * 
     * 
     * @return the custom path separators
     * @since 2.0c
     */
    public String getPathSeparators() {        
    	return pathSeparators;
    }                
        
    /**
     * <p>
     * Set the custom path separators, which should contain no duplicates (but can be empty).
     * </p>
     * 
     * 
     * @param pathSeparators the path separators to set
     * @throws NullPointerException if pathSeparators is null
     * @throws IllegalArgumentException if pathSeparators contains duplicates
     * @since 2.0c
     */
    public void setPathSeparators(String pathSeparators) {      
    	// check for null.
        if (pathSeparators == null) {
            throw new NullPointerException("pahSeparators should not be null.");
        }
        
        // check for duplicate characters.
        if (!isValidPathSeparators(pathSeparators)) {
        	throw new IllegalArgumentException("pathSeparators contains duplicates");
        }
                           
    }
    
    /**
     * <p>
     * Return whether the given separator is valid path separators.
     * </p>
     * 
     * @param pathSeparators the path separators to check
     * @return true if the given path separators doesn't contain duplicates, false otherwise 
     * @since 2.0c
     */
    private boolean isValidPathSeparators(String pathSeparators) {
    	// check for duplicate characters.
    	charSet = new HashSet();
        for (int i = 0; i < pathSeparators.length(); ++i) {
        	Character character = new Character(pathSeparators.charAt(i));
        	if (charSet.contains(character)) {
        		// return immediately if a duplicate found
        		return false;
        	}
        	charSet.add(character);
        }
                       
        this.pathSeparators = pathSeparators;                
        return true;    
    }
    
    /**
     * Get the parent directory of the give file.
     * 
     * @param filePath the path to parse
     * @return the parent directory of the give file.
     */
    private String getParent(String filePath) {
    	
    	for (int i = filePath.length() - 1; i >= 1; --i) {
    		if (filePath.charAt(i) == File.separatorChar || charSet.contains(new Character(filePath.charAt(i)))) {
    			return  filePath.substring(0, i);    			
    		}
    	}
    	return null;    		
    }

}
