/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.util.config.Property;

import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * BasicStatsReporting is a reporting implementation that renders user-friendly reports.
 * </p>
 *
 * <p>
 * Configuration: It does not support any configuration. The constructor is included to address
 * reflection contract.
 * </p>
 *
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class BasicStatsReporting extends AbstractStatsReporting {
    /**
     * <p>
     * Line separator character.
     * </p>
     */
    private static final char LINE_SEPARATOR = (char) Character.LINE_SEPARATOR;

    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public BasicStatsReporting() {
        // empty
    }

    /**
     * <p>
     * Constructor with property. This does not have any functionality in the current release and
     * is kept for extension purpose.
     * </p>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public BasicStatsReporting(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * This generates a user-readable report.
     * </p>
     *
     * @return the generated report
     */
    public String generateReport() {
        StringBuffer report = new StringBuffer();

        // delegate generateStats to render each stats
        generateStats("-= Files =-", getFileStats(), report);
        generateStats("-= Directories =-", getDirectoryStats(), report);
        generateStats("-= Aliases =-", getAliasStats(), report);
        generateStats("-= Ovelall =-", getOverallStats(), report);

        return report.toString();
    }

    /**
     * <p>
     * Generates the stats for a stats collection.
     * </p>
     *
     * @param title the title
     * @param stats the stats collection to render
     * @param report the StringBuffer containing the report
     */
    private void generateStats(String title, StatsCollection stats, StringBuffer report) {
        report.append(title).append(LINE_SEPARATOR);

        List results = stats.getAllKeys();

        // render all StatsResults
        for (Iterator itr = results.iterator(); itr.hasNext();) {
            String key = (String) itr.next();
            StatsResult result = stats.get(key);
            generateStats("- " + key, result, report);
        }

        report.append(LINE_SEPARATOR);
    }

    /**
     * <p>
     * Generates the stats for a StatsResult.
     * </p>
     *
     * @param title the title
     * @param stats the StatsResult to render
     * @param report the StringBuffer containing the report
     */
    private void generateStats(String title, StatsResult stats, StringBuffer report) {
        report.append(title).append(LINE_SEPARATOR);

        List metrics = stats.getMetrics();
        // render all metrics
        for (Iterator itr = metrics.iterator(); itr.hasNext();) {
            Metric metric = (Metric) itr.next();
            report.append(metric.getName()).append(": ").append(stats.getStat(metric));
            report.append(LINE_SEPARATOR);
        }
    }


}
