/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import junit.framework.TestCase;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;


/**
 * <p>
 * Unit test cases for ExtendedFileStatistics.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ExtendedFileStatisticsTests extends TestCase {
    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files";

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.unittest";

    /**
     * An Metric instance to test.
     */
    private Metric line = null;

    /**
     * An ExtendedFileStatistics instance to test.
     */
    private ExtendedFileStatistics fileStatistics = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor1 = new CStyleLineCounter("java", new ExtensionMatcher("java"));

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor2 = new TextLineCounter("text", new ExtensionMatcher("txt"));

    /**
     * Creates an ExtendedFileStatistics instance.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        fileStatistics = new ExtendedFileStatistics();

        line = (Metric) processor1.getSupportedMetrics().get(0);
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(String).
     * </p>
     */
    public void testExtendedFileStatistics1() {
        fileStatistics = new ExtendedFileStatistics();
        assertTrue("ExtendedFileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertTrue("Unable to instantiate ExtendedFileStatistics.",
            fileStatistics.getAllFileProcessors().isEmpty());
        assertFalse("Unable to instantiate ExtendedFileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(String) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics2() throws Exception {
        try {
            new ExtendedFileStatistics(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(String) with empty string.
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics3() throws Exception {
        try {
            new ExtendedFileStatistics(" ");
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics4() throws Exception {
        fileStatistics = createExtendedFileStatistics("config1.xml", NAMESPACE);

        assertTrue("ExtendedFileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate ExtendedFileStatistics.", 2,
            fileStatistics.getAllFileProcessors().size());
        assertTrue("Unable to instantiate ExtendedFileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics5() throws Exception {
        fileStatistics = createExtendedFileStatistics("config2.xml", NAMESPACE);

        assertTrue("ExtendedFileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate ExtendedFileStatistics.", 1,
            fileStatistics.getAllFileProcessors().size());
        assertTrue("Unable to instantiate ExtendedFileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics6() throws Exception {
        fileStatistics = createExtendedFileStatistics("config3.xml", NAMESPACE);

        assertTrue("ExtendedFileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate ExtendedFileStatistics.", 2,
            fileStatistics.getAllFileProcessors().size());
        assertFalse("Unable to instantiate ExtendedFileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics7() throws Exception {
        fileStatistics = createExtendedFileStatistics("config4.xml", NAMESPACE);

        assertTrue("ExtendedFileStatistics is instantiated incorrectly.",
            fileStatistics.getReporting() instanceof XmlStatsReporting);
        assertEquals("Unable to instantiate ExtendedFileStatistics.", 7,
            fileStatistics.getAllFileProcessors().size());

        assertFalse("Unable to instantiate ExtendedFileStatistics.", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Tests the constructor against invalid configurations.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics8() throws Exception {
        checkInvalidConfig("invalid1.xml");
        checkInvalidConfig("invalid2.xml");
        checkInvalidConfig("invalid3.xml");
        checkInvalidConfig("invalid4.xml");
        checkInvalidConfig("invalid5.xml");
        checkInvalidConfig("invalid6.xml");
        checkInvalidConfig("invalid7.xml");
    }

    /**
     * <p>
     * Verifies the constructor - ExtendedFileStatistics(String). ConfigurationException should be
     * thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtendedFileStatistics9() throws Exception {
        try {
            new ExtendedFileStatistics(NAMESPACE);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Creates a ExtendedFileStatistics instance with given config namespace.
     * </p>
     *
     * @param configFile the config file
     * @param namespace the config namespace
     *
     * @return the ExtendedFileStatistics instance
     *
     * @throws Exception to JUnit
     */
    private ExtendedFileStatistics createExtendedFileStatistics(String configFile, String namespace)
        throws Exception {
        TestHelper.loadConfigFile(configFile, namespace);

        ExtendedFileStatistics statistics = new ExtendedFileStatistics(namespace);
        TestHelper.removeNamespace(namespace);

        return statistics;
    }

    /**
     * Test the constructor against invalid configuration.
     *
     * @param configFile the invalid config file.
     *
     * @throws Exception to JUnit
     */
    private void checkInvalidConfig(String configFile)
        throws Exception {
        TestHelper.loadConfigFile(configFile, NAMESPACE);

        try {
            new ExtendedFileStatistics(NAMESPACE);
            fail("ConfigurationException should be thrown.");
        } catch (ConfigurationException e) {
            // success
        } finally {
            TestHelper.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Test addFileProcessor method with null argument. NullPointerException should be thrown.
     * </p>
     */
    public void testAddFileProcessor1() {
        try {
            this.fileStatistics.addFileProcessor(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test addFileProcessor method. The order should be maintained.
     * </p>
     */
    public void testAddFileProcessor2() {
        fileStatistics.addFileProcessor(processor1);
        assertEquals("processor1 should be added", processor1,
            fileStatistics.getAllFileProcessors().get(0));

        fileStatistics.addFileProcessor(processor2);
        assertEquals("processor2 should be added", processor2,
            fileStatistics.getAllFileProcessors().get(1));

        fileStatistics.addFileProcessor(processor1);
        assertEquals("the order is wrong", processor2, fileStatistics.getAllFileProcessors().get(0));
        assertEquals("the order is wrong", processor1, fileStatistics.getAllFileProcessors().get(1));
    }

    /**
     * <p>
     * Test removeFileProcessor method with null argument. NullPointerException should be thrown.
     * </p>
     */
    public void testRemoveFileProcessor1() {
        try {
            this.fileStatistics.removeFileProcessor(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test removeFileProcessor method.
     * </p>
     */
    public void testRemoveFileProcessor2() {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        fileStatistics.removeFileProcessor(processor1.getAlias());
        assertEquals("processor1 should be removed", 1, fileStatistics.getAllFileProcessors().size());
        assertEquals("processor1 should be removed", processor2,
            fileStatistics.getAllFileProcessors().get(0));

        fileStatistics.removeFileProcessor(processor2.getAlias());
        assertEquals("processor2 should be removed", 0, fileStatistics.getAllFileProcessors().size());
    }

    /**
     * <p>
     * Test getAllFileProcessors method. The order should be maintained.
     * </p>
     */
    public void testGetAllFileProcessors() {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);
        fileStatistics.addFileProcessor(processor1);

        assertEquals("the size should be 2", 2, fileStatistics.getAllFileProcessors().size());
        assertEquals("the order is wrong", processor2, fileStatistics.getAllFileProcessors().get(0));
        assertEquals("the order is wrong", processor1, fileStatistics.getAllFileProcessors().get(1));
    }

    /**
     * <p>
     * Test clearAllFileProcessors method.
     * </p>
     */
    public void testClearAllFileProcessors() {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);
        fileStatistics.clearAllFileProcessors();

        assertTrue("the size should be 0", fileStatistics.getAllFileProcessors().isEmpty());
    }

    /**
     * <p>
     * Test getReporting method. The order should be maintained.
     * </p>
     */
    public void testGetReporting() {
        StatsReporting reporting = new BasicStatsReporting();
        fileStatistics.setReporting(reporting);

        assertEquals("reporting should be returned", reporting, fileStatistics.getReporting());
    }

    /**
     * <p>
     * Test setReporting method with null argument. NullPointerException should be thrown.
     * </p>
     */
    public void testSetReporting1() {
        try {
            this.fileStatistics.setReporting(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test setReporting method. The order should be maintained.
     * </p>
     */
    public void testSetReporting2() {
        StatsReporting reporting = new BasicStatsReporting();
        fileStatistics.setReporting(reporting);

        assertEquals("reporting should be set", reporting, fileStatistics.getReporting());
    }

    /**
     * <p>
     * Test getReporting method. The order should be maintained.
     * </p>
     */
    public void testGetAndSetMatchAll() {
        fileStatistics.setMatchAll(true);
        assertTrue("true is expected", fileStatistics.isMatchAll());
        fileStatistics.setMatchAll(false);
        assertFalse("false is expected", fileStatistics.isMatchAll());
    }

    /**
     * <p>
     * Test processFile method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile1() throws Exception {
        try {
            this.fileStatistics.processFile(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processFile method. Process multiple files.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile2() throws Exception {
        fileStatistics = createExtendedFileStatistics("config3.xml", NAMESPACE);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processFile(new File(BASE_PATH, "test.java"));
        fileStatistics.processFile(new File(BASE_PATH, "test.txt"));
        fileStatistics.processFile(new File(BASE_PATH, "dir/test.java"));
        fileStatistics.processFile(new File(BASE_PATH, "dir/test.txt"));

        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("test.txt").getStat(line));
        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("dir\\test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("dir\\test.txt").getStat(line));
    }

    /**
     * <p>
     * Test processDirectory(File) with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory1() throws Exception {
        try {
            fileStatistics.processDirectory(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File) with invalid dir. IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory2() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "no_such_dir"));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File) with invalid dir. IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory3() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "test.txt"));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File, boolean) with null argument. NullPointerException should be
     * thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory4() throws Exception {
        try {
            fileStatistics.processDirectory(null, false);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File, boolean) with invalid dir. IllegalArgumentException should be
     * thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory5() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "no_such_dir"), false);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File, boolean) with invalid dir. IllegalArgumentException should be
     * thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory6() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "test.txt"), false);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory7() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"));

        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("dir\\test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("dir\\test.txt").getStat(line));
    }

    /**
     * <p>
     * Test processDirectory(File, boolean).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory8() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"), true);

        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("dir\\test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("dir\\test.txt").getStat(line));
    }

    /**
     * <p>
     * Test printReport().
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport1() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"), true);

        fileStatistics.printReport();
    }

    /**
     * <p>
     * Test printReport(File).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport2() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"), true);

        File output = new File(BASE_PATH, "output");
        fileStatistics.printReport(output);

        char[] buffer = new char[20000];
        BufferedReader reader = new BufferedReader(new FileReader(output));
        int length = reader.read(buffer);

        assertEquals("the result is incorrect", reporting.generateReport(),
            new String(buffer, 0, length));

        output.delete();
    }

    /**
     * <p>
     * Test printReport(File) with null dir. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport3() throws Exception {
        try {
            fileStatistics.printReport(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test printReport(File) with invalid dir. IOException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport4() throws Exception {
        try {
            fileStatistics.printReport(new File(BASE_PATH));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }
}
