/*
 * $(#)MimeMatcherTests.java		Sep 6, 2005
 * 
 * Copyright (c) 2005, TopCoder Inc. All rights reserved.
 */

package com.topcoder.file.statistics.failuretests.matcher;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.matcher.MimeMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

import junit.framework.TestCase;

/**
 * This test case contains the failure tests for the <code>MimeMatcher</code>
 * class.
 *
 * @author MPhk
 * @version 1.0
 */
public class MimeMatcherTests extends TestCase {

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.matcher.failuretests";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "failuretests/matcher/config.xml";

    /**
     * The namespace for Magic Numbers component.
     */
    private static final String MAGIC_NUMBERS_NAMESPACE = "com.topcoder.file.type.MagicNumbers.xml";

    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/failuretests/matcher/";

    /**
     * A MimeMatcher instance to test.
     */
    private MimeMatcher matcher = null;

    /**
     * Load the configuration before testing.
     *
     * @throws Exception in case anything goes wrong
     */
    protected void setUp() throws Exception {
        // clear the namespace first
        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
        // load the configuration
        ConfigManager.getInstance().add(CONFIG_FILE);
        
        if (configManager.existsNamespace(MAGIC_NUMBERS_NAMESPACE)) {
            configManager.removeNamespace(MAGIC_NUMBERS_NAMESPACE);
        }
        matcher = new MimeMatcher("image/gif");
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception in case anything goes wrong
     */
    protected void tearDown() throws Exception {
        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
        if (configManager.existsNamespace(MAGIC_NUMBERS_NAMESPACE)) {
            configManager.removeNamespace(MAGIC_NUMBERS_NAMESPACE);
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(Pattern) properly throws
     * NullPointerException for null argument.
     */
    public void testMimeMatcher1() {
        try {
            new MimeMatcher((String) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }
    
    /**
     * Checks whether the constructor MimeMatcher(Pattern) properly throws
     * IllegalArgumentException for empty Stng argument.
     */
    public void testMimeMatcher2() {
        try {
            new MimeMatcher("");
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(Property) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher3() throws Exception {
        try {
            new MimeMatcher((Property) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(Property) properly throws
     * Configuration for invalid property - no values in "mimes" sub-property.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher4() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "mimes_invalid1");

        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(Property) properly throws
     * Configuration for invalid property - no "mimes" sub-property.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher5() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "mimes_invalid2");

        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(Property) properly throws
     * Configuration for invalid property - empty values.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher6() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "mimes_invalid3");

        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }
    
    /**
     * Checks whether the constructor MimeMatcher(List) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher7() throws Exception {
        try {
            new MimeMatcher((List) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(List) properly throws
     * IllegalArgumentException for empty List argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher8() throws Exception {
        try {
            new MimeMatcher(new ArrayList());
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(List) properly throws
     * IllegalArgumentException for List argument containing nulls.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher9() throws Exception {
        List list = new ArrayList();
        list.add("1");
        list.add(null);

        try {
            new MimeMatcher(list);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(List) properly throws
     * IllegalArgumentException for List argument containing empty Strings.
     *
     * @throws Exception in case anything goes wrong
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher10() throws Exception {
        List list = new ArrayList();
        list.add("1");
        list.add("");

        try {
            new MimeMatcher(list);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor MimeMatcher(List) properly throws
     * IllegalArgumentException for List argument containing non String elements.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMimeMatcher11() throws Exception {
        List list = new ArrayList();
        list.add("1");
        list.add(new Object());

        try {
            new MimeMatcher(list);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Tests the match method with null argument. NullPointerException should be thrown.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMatch1() throws Exception {
        try {
            matcher.match((File) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Tests the match method for inexistent file. IOException should be thrown.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMatch2() throws Exception {
        try {
            matcher.match(new File(BASE_PATH, "inexistent_file"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }

    /**
     * Tests the match method for a folder. IOException should be thrown.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMatch3() throws Exception {
        try {
            matcher.match(new File(BASE_PATH));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }
    
}
