/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;
import junit.framework.TestCase;
import java.io.File;
import java.io.IOException;
import java.util.List;


/**
 * <p>
 * Unit test cases for TextLineCounter.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TextLineCounterTests extends TestCase {
    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.processor.linecounter.unittest";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "processor/processor.xml";

    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/processor/";

    /**
     * The the Metric instance for test.
     */
    private Metric metric = null;

    /**
     * An TextLineCounter instance to test.
     */
    private TextLineCounter counter = null;

    /**
     * Load the configuration before testing.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        TestHelper.loadConfigFile(CONFIG_FILE, NAMESPACE);
        counter = new TextLineCounter("foo", new ExtensionMatcher("foo"));
        metric = (Metric) counter.getSupportedMetrics().get(0);
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(String, FileMatcher).
     * </p>
     */
    public void testTextLineCounter1() {
        FileMatcher matcher = new ExtensionMatcher("foo");
        counter = new TextLineCounter("foo", matcher);
        assertTrue("Unable to instantiate TextLineCounter.", counter instanceof FileProcessor);
        assertEquals("alias is incorrect", "foo", counter.getAlias());
        assertEquals("match is incorrect", matcher, counter.getFileMatcher());
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(String, FileMatcher) with null argument.
     * NullPointerException should be thrown.
     * </p>
     */
    public void testTextLineCounter2() {
        try {
            new TextLineCounter(null, new ExtensionMatcher("foo"));
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(String, FileMatcher) with null argument.
     * NullPointerException should be thrown.
     * </p>
     */
    public void testTextLineCounter3() {
        try {
            new TextLineCounter("foo", null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(String, FileMatcher) with empty string.
     * IllegalArgumentException should be thrown.
     * </p>
     */
    public void testTextLineCounter4() {
        try {
            new TextLineCounter(" ", new ExtensionMatcher("foo"));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter5() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "line_counter");

        counter = new TextLineCounter(property);
        assertTrue("Unable to instantiate TextLineCounter.", counter instanceof FileProcessor);
        assertEquals("alias is incorrect", "foo", counter.getAlias());

        FileMatcher matcher = counter.getFileMatcher();
        assertTrue("the matcher is incorrect", matcher.match(new File("bar.foo")));
        assertFalse("the matcher is incorrect", matcher.match(new File("foo.bar")));
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter6() throws Exception {
        try {
            new TextLineCounter((Property) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter7() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid1");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter8() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid2");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter9() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid3");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property).
     * The property contains no matcher, an AnyFileMatcher should be used.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter10() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "no_matcher");
        assertTrue("the matcher should be type of AnyFileMatcher.",
                new TextLineCounter(property).getFileMatcher() instanceof AnyFileMatcher);
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter11() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid5");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter12() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid6");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter13() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid7");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter14() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid8");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - TextLineCounter(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testTextLineCounter15() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "line_counter_invalid9");

        try {
            new TextLineCounter(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the getAlias method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetAlias() throws Exception {
        assertEquals("the alias is incorrect", "foo", counter.getAlias());
    }

    /**
     * <p>
     * Tests the getFileMatcher method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetFileMatcher() throws Exception {
        assertTrue("the matcher is incorrect", counter.getFileMatcher() instanceof ExtensionMatcher);
    }

    /**
     * <p>
     * Tests the getSupportedMetrics method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetSupportedMetrics() throws Exception {
        List metrics = counter.getSupportedMetrics();
        assertEquals("The size should be 1.", 1, metrics.size());

        metric = (Metric) metrics.get(0);
        assertEquals("The metric is incorrect.", "Line Count", metric.getName());
        assertEquals("The metric is incorrect.", "line", metric.getDisplayName());
    }

    /**
     * <p>
     * Tests the process method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws IOException to JUnit
     */
    public void testProcess1() throws IOException {
        try {
            counter.process(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the process method with invalid argument. IOException should be thrown.
     * </p>
     */
    public void testProcess2() {
        try {
            counter.process(new File(BASE_PATH));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the process method with invalid argument. IOException should be thrown.
     * </p>
     */
    public void testProcess3() {
        try {
            counter.process(new File(BASE_PATH, "no_such_file"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the process method with a text file.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcess4() throws Exception {
        StatsResult result = counter.process(new File(BASE_PATH, "10lines.txt"));
        assertEquals("The result is incorrect.", 10, result.getStat(metric));
    }

    /**
     * <p>
     * Tests the process method with an empty file.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcess5() throws Exception {
        StatsResult result = counter.process(new File(BASE_PATH, "empty"));
        assertEquals("The result is incorrect.", 0, result.getStat(metric));
    }
}
